function Remove-DenyWriteAcl {
  [CmdletBinding(SupportsShouldProcess=$true,ConfirmImpact="Medium")]
  param(
    [string] $targetDir = ''
  )
  if($PSCmdlet.ShouldProcess($targetDir)){
    $acl = Get-Acl $targetDir
    $inheritanceFlag = [System.Security.AccessControl.InheritanceFlags]::ContainerInherit -bor [System.Security.AccessControl.InheritanceFlags]::ObjectInherit
    $propagationFlag = [System.Security.AccessControl.PropagationFlags]::None
    $permType = [System.Security.AccessControl.AccessControlType]::Deny

    $permission = "everyone","write",$inheritanceFlag,$propagationFlag,$permType
    $accessRule = New-Object System.Security.AccessControl.FileSystemAccessRule $permission
    $acl.RemoveAccessRule($accessRule)
    $acl | Set-Acl $targetDir
  }
}

$progData =  [System.Environment]::GetEnvironmentVariable('ProgramData')
$targetFolder = Join-Path $progData "osquery"
$daemonFolder = Join-Path $targetFolder "osqueryd"

# Before we install, we check to see if the package is already installed
# as we need to remove the DenyWrite ACLs required by osqueryd, along with
# stopping the service.
if ((Get-Service "osqueryd" -ErrorAction SilentlyContinue) -and (Get-Service "osqueryd").Status -eq 'Running') {
  Stop-Service osqueryd
  Set-Service osqueryd -startuptype "manual"
  Get-CimInstance -ClassName Win32_Service -Filter "Name='osqueryd'" | Invoke-CimMethod -methodName Delete
}

if (Test-Path $daemonFolder) {
  Remove-DenyWriteAcl $daemonFolder
}
